MSP430 Base Station
Copyright (C) 2007 Joseph Bosman, Steve Olivieri, Ipek Ozil, Brandon Steacy

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA


#include <msp430x16x.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef BASE_STATION_H
#define BASE_STATION_H

/* nHOST_INT interrupt boolean */
volatile unsigned char nHOST_INT;

/* Tracks potential callbacks for the EM260 */
volatile unsigned char potentialCallback;

/* Interrupt boolean for the PC */
volatile unsigned char pcINT;

/* Buffer for storing commands from the PC */
unsigned char pcBuff[150];

/* Number of characters currently in pcBuff */
volatile unsigned char pcI;

/* Node ID of the last IMAGE_READY signal */
unsigned char irID[2];

/* IMAGE_READY boolean */
unsigned char imageReady;

/* Interrupt routines. */
#ifdef __IAR_SYSTEMS_ICC__
  #pragma vector=USART0RX_VECTOR
    __interrupt void usart0_rx(void);
  #pragma vector=PORT1_VECTOR
    __interrupt void em260(void);
#else
  interrupt (USART0RX_VECTOR) usart0_rx(void);
  interrupt (PORT1_VECTOR) em260(void);
#endif

/* Wait for a while.  Specifically, perform two nop() opertions for each
 * unit of 'time.'  How long is a unit?  About 6 operations in GCC without
 * optimizations.  This is a rather poor way of waiting, but it'll do for
 * now.
 * 	@param time: the number of units of time to wait
 */
void delay(unsigned int time);

/* Initializes UART0 on Port 3 of the MSP430. */
void initUART0(void);

/* Initializes SPI1 on Port 5 of the MSP430. */
void initSPI1(void);

/* Initializes the EM260 by performing the following tasks:
 *	- resets the EM260
 *	- performs a reset transaction over SPI
 *	- performs a "verion" transaction over SPI
 *	- performs a "SPI status" transaction over SPI
 */
unsigned int initEM260(void);

/* Cleans up the MSP430 after a reboot by performing the following tasks:
 * 	- sets all I/O pins to I/O mode
 * 	- sets all I/O pins to output
 * 	- sets all I/O pins to low
 * 	- turns off all interrupts
 * 	- sets interrupt flags to low
 *	- sets interrupts modes to low-to-high
 */
void initValues(void);

/* Initializes the MSP430 by performing the following tasks:
 *	- sets the MSP to use the 8MHz crystal at 4MHz
 *	- turns off the watchdog timer
 *	- globally enables interrupts
 *	- clears registers (see initValues())
 *	- sets up pins for EM260 operation
 *	- clears nHOST_INT
 */
void initMSP(void);

/* Sends a string to the EM260 over SPI1.
 * 	@param buff: a pointer to the string to send
 * 	@param len: the length of the string
 */
void toEM260(unsigned char* buff, unsigned int len);

/* Receives data from the EM260 over SPI1.
 * 	@param buff: a pointer to the buffer to store the response in
 * 	@return: the number of bytes received (and stored in buff)
 */
unsigned char fromEM260(unsigned char* buff);

/* Sends a string to the PC over UART0.
 * 	@param buff: a pointer to the string to send
 * 	@param len: the length of the string
 */
void toPC(unsigned char* buff, unsigned int len);

/* Establishes a ZigBee PAN with this EM260 as the coordinator.
 * The extended PAN ID is NNNNNNNNNNNNNNNN and the PAN ID is NNNN.
 * The network exists on Channel 13.  This function uses EZSP to
 * call the emberFormNetwork() command.
 * 	@param sq: the sequence number of the EZSP frame
 * 	@return: the status of the network
 */
unsigned char EM260_establishNetwork(unsigned char sq);

/* Sets a configuration value on the EM260.  The params buffer
 * must contain the following: EZSP sequence number (1 byte),
 * the ConfigId (1 byte), and the new value for that ConfigId
 * (2 bytes).  ConfigIds are specified in the EM260 data sheet.
 * This function returns the status value from the EZSP response.
 * 	@param params: the arguments for setConfigurationValue()
 * 	@return: the status of the command
 */
unsigned char EM260_setConfigurationValue(unsigned char* params);

/* Calls the EZSP networkInit() function.  The result of calling
 * this function depends on whether or not the EM260 was joined
 * to a network on last power off.  If so, a callbac must follow
 * this command.  In addition, the EM260 will once again be on
 * that network.  If not, no callback should be issued and the
 * EM260 will not be joined to any networks.
 * 	@param sq: the sequence number of the EZSP frame
 * 	@return: the network status
 */
unsigned char EM260_networkInit(unsigned char sq);

/* Calls the EZSP leaveNetwork() function.  The result of this
 * call is the return value for EM260_leaveNetwork().  If this
 * call is successful, the EM260 will leave any network that it
 * is joined to.  Else, the state of the network is unknown.
 * 	@param sq: the sequence number of the EZSP frame
 * 	@return: the status of the command
 */
unsigned char EM260_leaveNetwork(unsigned char sq);

/* Forms a network via the EZSP scanAndFormNetwork() command.
 * The extended PAN ID of the network is F1332548CA02121F,
 * the short PAN ID is chosen randomly, and the Channel is
 * automatically selected after scanning all available channels
 * and determining which one has the least amount of interference.
 * After issuing this command, a callback must be made.
 * 	@param sq: the sequence number of the EZSP frame
 */
void EM260_formNetwork(unsigned char sq);

/* Permits nodes to join the network (as routers or end-devices).
 * This function must be called after EM260_formNetwork() or else
 * the network will not function properly.
 * 	@param sq: the sequence number for the EZSP frame
 * 	@return: the status of the command
 */
unsigned char EM260_permitJoining(unsigned char sq);

/* Gets the parameters of the network that the EM260 is joined to.
 * Since the EM260 should always be acting as a coordinator, it
 * is not necessary to obtain the NodeId (which is always 0000).
 * The PAN ID is stored in bytes 15-16 (little endian), the channel
 * is stored in byte 18 of buff.  The extended PAN ID is stored
 * in bytes 7-14 of buff.
 * 	@param buff: pointer to the buffer to store the parameters in
 * 	@param sq: the sequence number for the EZSP frame
 */
void EM260_getNetworkParameters(unsigned char* buff, unsigned char sq);

/* Handles callbacks from the EM260.  Currently, it ignores all
 * callbacks except for the incomingMessageHandler callback as
 * that's all we really care about.
 * 	@param buff: the callback to handle (result of EM260_callback())
 * 	@param sq: the sequence number of any EZSP frames that we send
 */
void handleCallback(unsigned char* buff, unsigned char sq);

/* Issues a callback() command to the EM260 and receives a response.
 * Typically, callbacks are made after calling other commands.  In the
 * case that the EM260 must send us a message asynchronously, the
 * nHOST_INT interrupt will be triggered and a callback must be made.
 * 	@param buff: pointer to the buffer to store the respnse in
 * 	@param sq: the sequence number for the EZSP frame
 */
unsigned char EM260_callback(unsigned char* buff, unsigned char sq);

/* Sends a unicast message to another node on the network via the
 * EZSP sendUnicast() function.  If this function is successful,
 * a callback must follow.  This function always uses ProfileId 0,
 * ClusterId 0, and EndPoint 0x01 (source and destination).  Messages
 * are always tagged with 0x64, route discovery is enabled, retries
 * are enabled, and the source EUI64 is always sent before the message.
 * 	@param addr: the two-byte NodeID to send the message to
 * 	@param message: the message to send
 * 	@param len: the length of message
 * 	@param sq: the sequence number for the EZSP frame
 * 	@param sq2: the sequence number for the unicast
 */
unsigned char EM260_sendUnicast(unsigned char* addr, unsigned char* message,
	unsigned char len, unsigned char sq, unsigned char sq2);
	
#endif /* BASE_STATION_H */
